// --- Saved variables ---
// You can add what you want to the save
savedInt 		= 0; // Here is the initialization for the first launch, after that the value will depend on what has been saved
currentWave     = 0;

function saveState() {
	state.scriptProps = {
		savedInt 		: savedInt,
		currentWave     : currentWave,
	};
}


// --- Local variables ---
var waveSpeed = 180;
//var spawnerZonesID = [240, 196, 198, 141, 84, 118, 97, 130];
//var playerZonesID  = [265, 243, 225, 165, 57, 152, 43, 187];
var islandZonesID = [ [265, 251, 276, 241, 252, 262, 255, 249, 226, 240],
					[243, 236, 238, 219, 232, 217, 204, 207, 203, 196],
					[225, 239, 213, 218, 230, 205, 222, 211, 195, 198],
					[165, 171, 144, 157, 167, 149, 136, 154, 134, 141],
					[ 57,  72,  52,  87,  66,  59,  96,  78,  67, 84],
					[152, 133, 158, 147, 139, 121, 137, 128, 116, 118],
					[ 43,  49,  53,  63,  54,  64,  83,  74,  88,  97],
					[187, 177, 178, 159, 164, 160, 142, 151, 135, 130]];

//var spawnerZones:Array<Zone>;
//var playerZones:Array<Zone>;
var islandZones:Array<Array<Zone>> =  [for(i in 0...islandZonesID.length) [for(j in 0...islandZonesID[i].length) getZone(islandZonesID[i][j]) ]];

// --- Script code ---
function init() {
	if (state.time == 0)
		onFirstLaunch();
	onEachLaunch();
}

function onFirstLaunch() {
	removeUnwantedVictories();
	preventColonizationOfSpawnerTiles();

	//keep this for testing
	for(player in state.players){
		player.discoverAll();
	}

}

function onEachLaunch() {

}

// Regular update is called every 0.5s
function regularUpdate(dt : Float) {
	 if(toInt(state.time / waveSpeed) > currentWave) {
		 currentWave++;
		 waveSpawner();
		 checkForDeadPlayers();

		//give AI players some resources each wave to hopefully make it easier for them
		for(player in state.players){
			if(player.isAI){
				player.addResource(Resource.Fame, randomInt(61) + 30, true);
				player.addResource(Resource.Lore, randomInt(121) + 30, true);
				player.addResource(Resource.Food, randomInt(121) + 30, true);
				player.addResource(Resource.Wood, randomInt(121) + 30, true);
				player.addResource(Resource.Money, randomInt(121) + 30, true);
				player.addResource(Resource.MilitaryXP, randomInt(121) + 30, true);
				player.addResource(Resource.Stone, randomInt(5) + 1, true);
				player.addResource(Resource.Iron, randomInt(5) + 1, true);
			}
		}
	 }

	 //for now only continues attacks from units stuck in the spawner area
	if(randomInt(10) + 1 == 10)
		continueAttacks();
}

function removeUnwantedVictories() {
	//In Gate Defense you win by being the last clan alive. (aka Military)
	state.removeVictory(VictoryKind.VFame);
	state.removeVictory(VictoryKind.VHelheim);
	state.removeVictory(VictoryKind.VLore);
	state.removeVictory(VictoryKind.VMoney);
	state.removeVictory(VictoryKind.VOdinSword);
	state.removeVictory(VictoryKind.VYggdrasil);
}

function preventColonizationOfSpawnerTiles() {
	for(player in state.players)
		for(i in 0...islandZones.length) {
			player.allowColonize(islandZones[i][9], false);
		}
}

function checkForDeadPlayers() {
	var i = 0;
	while(i < islandZones.length) {
		if(islandZones[i][0].owner == null){
			killAllUnits(islandZones[i]);
			islandZones.remove(islandZones[i]);
			break;
		}
		else
			i++;

		wait(0);
	}
}

function continueAttacks() {
	for(i in 0...islandZones.length) {
		if(islandZones[i][9].units.length > 0) {
			launchAttackPlayer(islandZones[i][9].units, islandZones[i][0].owner);
			wait(0);
		}
	}
}

function launchAttacks(u:UnitKind, amount) {
	for(i in 0...islandZones.length) {
		launchAttackPlayer(islandZones[i][9].addUnit(u, amount), islandZones[i][0].owner);
	}
}

function waveSpawner() {

	//spawns wolves from waves 1 to 12
	if(currentWave == 1) {
		launchAttacks(Unit.Wolf, 1);
	}
	else if(currentWave == 2) {
		launchAttacks(Unit.Wolf, 2);
	}
	else if(currentWave >= 3 && currentWave <= 12) {
		launchAttacks(Unit.Wolf, 3);
	}

	//spawns white wolves from waves 4 to 15
	if(currentWave == 4) {
		launchAttacks(Unit.WhiteWolf, 1);
	}
	else if(currentWave == 5) {
		launchAttacks(Unit.WhiteWolf, 2);
	}
	else if(currentWave >= 6 && currentWave <= 15) {
		launchAttacks(Unit.WhiteWolf, 3);
	}

	//spawns draugr from waves 7 to 18
	if(currentWave == 7) {
		launchAttacks(Unit.Death, 1);
	}
	else if(currentWave == 8) {
		launchAttacks(Unit.Death, 2);
	}
	else if(currentWave >= 9 && currentWave <= 18) {
		launchAttacks(Unit.Death, 3);
	}

	//spawn valkyries from waves 10 to 21
	if(currentWave == 10) {
		launchAttacks(Unit.Valkyrie, 1);
	}
	else if(currentWave == 11) {
		launchAttacks(Unit.Valkyrie, 2);
	}
	else if(currentWave >= 12 && currentWave <= 21) {
		launchAttacks(Unit.Valkyrie, 3);
	}

	//spawn spectral warriors from waves  13 to 24
	if(currentWave == 13) {
		launchAttacks(Unit.SpecterWarrior, 1);
	}
	else if(currentWave == 14) {
		launchAttacks(Unit.SpecterWarrior, 2);
	}
	else if(currentWave >= 15 && currentWave <= 24) {
		launchAttacks(Unit.SpecterWarrior, 3);
	}

	//spawn white & brown bears from waves 16 to 27
	if(currentWave == 16) {
		launchAttacks(Unit.Bear, 1);
		launchAttacks(Unit.WhiteBear, 1);
	}
	else if(currentWave == 17) {
		launchAttacks(Unit.Bear, 2);
		launchAttacks(Unit.WhiteBear, 2);
	}
	else if(currentWave >= 18 && currentWave <= 27) {
		launchAttacks(Unit.Bear, 3);
		launchAttacks(Unit.WhiteBear, 3);
	}

	//spawn rock golems from waves 19 to 30
	if(currentWave == 19) {
		launchAttacks(Unit.Golem, 1);
	}
	else if(currentWave == 20) {
		launchAttacks(Unit.Golem, 2);
	}
	else if(currentWave >= 21 && currentWave <= 30) {
		launchAttacks(Unit.Golem, 3);
	}

	//spawn colossal boar from waves 22 to 33
	if(currentWave == 22) {
		launchAttacks(Unit.ColossalBoar, 1);
	}
	else if(currentWave == 23) {
		launchAttacks(Unit.ColossalBoar, 2);
	}
	else if(currentWave >= 24 && currentWave <= 33) {
		launchAttacks(Unit.ColossalBoar, 3);
	}

	//spawn dark vaettirs & eldthurs from waves 25 to 36
	if(currentWave == 25) {
		launchAttacks(Unit.IceGolem, 1);
		launchAttacks(Unit.GiantFire, 1);
	}
	else if(currentWave == 26) {
		launchAttacks(Unit.IceGolem, 2);
		launchAttacks(Unit.GiantFire, 2);
	}
	else if(currentWave >= 27 && currentWave <= 36) {
		launchAttacks(Unit.IceGolem, 3);
		launchAttacks(Unit.GiantFire, 3);
	}
}